#ifndef DUCHAMP_DATA_ARRAY_BASE_H_
#define DUCHAMP_DATA_ARRAY_BASE_H_

#include <iostream>
#include <duchamp/duchamp.hh>
#include <vector>

namespace duchamp
{
    template <class T> 
    class DataArrayBase
    {
    public:
	DataArrayBase<T>();
	DataArrayBase<T>(std::vector<size_t> dim);
	DataArrayBase<T>(const DataArrayBase<T>& other);
	DataArrayBase<T>& operator= (const DataArrayBase<T>& other);
	virtual ~DataArrayBase<T>();

	T *data(){return itsArray;};
	void setArray(T *arr){itsArray=arr;}; // share the storage
	void copyArray(T *arr); // deep copy of array

	std::vector<size_t> dim(){return itsDim;};
	size_t ndim(){return itsDim.size();};
	size_t size(){return itsNumPixels;};

	T operator()(size_t pix);
	T operator()(size_t *pix);
	T operator()(std::vector<size_t> pix); // make pass-by-value, so we can allow for things like casa::IPosition::asStdVector()

	void operator+=(T val);
	void operator+=(DataArrayBase<T> &other);
	void operator*=(T val);
	void operator*=(DataArrayBase<T> &other);
	void operator-=(T val);
	void operator-=(DataArrayBase<T> &other);
	void operator/=(T val);
	void operator/=(DataArrayBase<T> &other);

	DataArrayBase<T> operator+(DataArrayBase<T> &other);
	DataArrayBase<T> operator-(DataArrayBase<T> &other);
	DataArrayBase<T> operator*(DataArrayBase<T> &other);
	DataArrayBase<T> operator/(DataArrayBase<T> &other);

	void throwInitErr(){DUCHAMPTHROW("DataArrayBase", "Array not initialised");};

    protected:
    
	void setup();
	size_t pixArrayToPos(size_t *pix);
	size_t pixVecToPos(std::vector<size_t> pix);
    
	T *itsArray;
	std::vector<size_t> itsDim;
	size_t itsNumPixels;
    
    };

}

#include <duchamp/DataArrays/DataArrayBase.tcc>

#endif
